/**************************************************************************
esPlaylist_COMPLETO.cpp

  PRIMA PARTE:
  Programmare le operazioni che gestiscono una struttura dati playlist. 
  Una playlist contiene una lista di album musicali. Ogni album musicale 
  e' caratterizzato da un titolo univoco di lunghezza massima 63 
  caratteri. La playlist non può contenere due album con lo stesso nome.
  All'inizio la playlist è vuota.

  Utilizzare la seguente struttura dati:

  struct elem {
	  char titolo[64];
	  elem* pun;
  };

  Realizzare le seguenti funzioni:

- stampa(L): funzione che stampa tutti gli album contenuti nella playlist
  L, rappresentata dal suo puntatore di testa.

- aggiungi(L, alb): funzione che aggiunge l'album di nome alb alla 
  playlist L. Un album il cui nome è più lungo di 63 caratteri o gia'
  presente nella lista non viene inserito. La funzione restituisce 1 
  se alb viene inserito correttamente in L, 0 altrimenti.

  SECONDA PARTE:
  Aggiungere le seguenti funzioni al programma:
- elimina(L, alb): funzione che elimina l'album di nome alb dalla 
  playlist L. La funzione restituisce 1 se alb viene eliminato
  correttamente da L, cioè se alb era presente nella playlist.
  Altrimenti, restituisce 0.
- cerca(L, alb): funzione che cerca l'album di nome alb all'interno 
  della playlist L. Se alb viene trovato, la funzione restituisce 1.
  Altrimenti, restituisce 0.
- distruggi(L): funzione che cancella tutto il contenuto della playlist L.

Testare le funzioni di prima e seconda parte con il seguente main:

  Utilizzare il seguente main di prova:

int main() {

	elem* L0;

	L0 = nullptr; // Playlist vuota
	stampa(L0);

	//test aggiungi
	aggiungi(L0, "titolo1 (autore1)");
	aggiungi(L0, "titolo2 (autore1)");
	aggiungi(L0, "titolo3 (autore2)");
	aggiungi(L0, "titolo2 (autore1)"); //non viene aggiunto perche'
	                                   //gia' presente
	stampa(L0);

	//test elimina
	elimina(L0, "titolo2 (autore1)");
	elimina(L0, "titolo8 (autore1)"); //non viene rimosso perche' 
	                                  //non presente
	stampa(L0);

	//test cerca
	cout << cerca(L0, "titolo1 (autore1)") << endl; //visualizza 1
	cout << cerca(L0, "titolo2 (autore1)") << endl; //visualizza 0

	//test distruggi
	distruggi(L0);
	stampa(L0);

	return 0;
}

  Output del main di test:
Album della playlist:

Album della playlist:
titolo3 (autore2)
titolo2 (autore1)
titolo1 (autore1)

Album della playlist:
titolo3 (autore2)
titolo1 (autore1)

1
0
Album della playlist:

**************************************************************/

#include <iostream>
#include <cstring>

using namespace std;

struct elem {
	char titolo[64];
	elem* pun;
};

void stampa(elem* L) {
	elem* p = L;
	cout << "Album della playlist:" << endl;
	while(p != nullptr) {
		cout << p->titolo<<endl;
		p = p->pun;
	}
	cout << endl;
}

bool aggiungi(elem*& L, const char* alb) {
	//controllo se nome troppo lungo
	if(strlen(alb) > 63) { return false; }
	//controllo se album gia' presente
	elem* p = L;
	while(p != nullptr) {
		if(strcmp(p->titolo,alb) == 0) { return false; }
		p = p->pun;
	}
	//inserimento in testa
	p = L;
	L = new elem;
	strcpy(L->titolo, alb);
	L->pun = p;
	return true;
}

bool elimina(elem*& L, const char* alb) {
	elem* p = L;
	elem* q = p;
	while(q != NULL && strcmp(q->titolo, alb) != 0) {
		p = q;
		q = q->pun;
	}
	if(q != nullptr) {
		if(q == L) {
			L = L->pun;
			delete q;
		}
		else {
			p->pun = q->pun;
			delete q;
		}
		return true;
	}
	return false;
}

bool cerca(elem* L, const char* alb) {
	elem* p = L;
	while(p != nullptr) {
		if(strcmp(p->titolo, alb) == 0) { return true; }
		p = p->pun;
	}
	return false;
}

void distruggi(elem*& L) {
	while(L != nullptr) {
		elem* p = L;
		L = L->pun;
		delete p;
	}
}

int main() {

	elem* L0;
	L0 = nullptr; // Playlist vuota
	stampa(L0);

	//test aggiungi
	aggiungi(L0, "titolo1 (autore1)");
	aggiungi(L0, "titolo2 (autore1)");
	aggiungi(L0, "titolo3 (autore2)");
	aggiungi(L0, "titolo2 (autore1)"); //non viene aggiunto perche' gia' presente
	stampa(L0);

	//test elimina
	elimina(L0, "titolo2 (autore1)");
	elimina(L0, "titolo8 (autore1)"); //non viene rimosso perche' non presente
	stampa(L0);

	//test cerca
	cout << cerca(L0, "titolo1 (autore1)") << endl; //visualizza 1
	cout << cerca(L0, "titolo2 (autore1)") << endl; //visualizza 0

	//test distruggi
	distruggi(L0);
	stampa(L0);

	return 0;
}
